<?php
if ( ! defined( 'ABSPATH' ) ) exit;

//Replace values in a provided variable with theme options, if available.
if ( ! function_exists( 'floris_get_option' ) ) {
	function floris_get_option( $var, $default = false ) {
		global $floris_opt;

		if ( isset( $floris_opt[ $var ] ) ) {
			$var = $floris_opt[ $var ];
		} else {
			$var = $default;
		}

		return $var;
	} // End floris_get_option()
}

//Multiselect function
 if ( ! function_exists( 'floris_element_values' ) ) {
    function floris_element_values(  $type = '', $query_args = array() ) {
      $options = array();
      switch( $type ) {
        case 'pages':
        case 'page':
        $pages = get_pages( $query_args );
        if ( !empty($pages) ) {
          foreach ( $pages as $page ) {
            $options[$page->post_title] = $page->ID;
          }
        }
        break;
        case 'posts':
        case 'post':
        $posts = get_posts( $query_args );
        if ( !empty($posts) ) {
          foreach ( $posts as $post ) {
            $options[$post->post_title] = lcfirst($post->post_title);
          }
        }
        break;
        case 'tags':
        case 'tag':
        $tags = get_terms( $query_args['taxonomies'], $query_args['args'] );
          if ( !empty($tags) ) {
            foreach ( $tags as $tag ) {
              $options[$tag->name] = $tag->slug;
          }
        }
        break;
        case 'categories':
        case 'category':
        $categories = get_categories( $query_args );
        if ( !empty($categories) ) {
          if(is_array($categories)){
          $options = array('All');
            foreach ( $categories as $category ) {
              if (is_object($category)) {
                $options[$category->name] = $category->slug;
              }
            }
          }
        }
        break;
        case 'products':
        case 'product':
        $loop = new WP_Query( $query_args );
        if ( !empty($loop) ) {
            while ( $loop->have_posts() ) : $loop->the_post(); 
              global $post; 
              if (is_object($post)) {
                $options[$post->post_title] = $post->ID;
              }
            endwhile; 
            wp_reset_postdata();
        } 
        break;
        case 'custom':
        case 'callback':
        if( is_callable( $query_args['function'] ) ) {
          $options = call_user_func( $query_args['function'], $query_args['args'] );
        }
        break;
      }
      return $options;
    }
}
if ( ! function_exists( 'floris_element_cat' ) ) {
    function floris_element_cat(  $type = '', $query_args = array() ) {
      $options = array();
      switch( $type ) {
        case 'pages':
        case 'page':
        $pages = get_pages( $query_args );
        if ( !empty($pages) ) {
          foreach ( $pages as $page ) {
            $options[$page->post_title] = $page->ID;
          }
        }
        break;
        case 'posts':
        case 'post':
        $posts = get_posts( $query_args );
        if ( !empty($posts) ) {
          foreach ( $posts as $post ) {
            $options[$post->post_title] = lcfirst($post->post_title);
          }
        }
        break;
        case 'tags':
        case 'tag':
        $tags = get_terms( $query_args['taxonomies'], $query_args['args'] );
          if ( !empty($tags) ) {
            foreach ( $tags as $tag ) {
              $options[$tag->name] = $tag->slug;
          }
        }
        break;
        case 'categories':
        case 'category':
        $categories = get_categories( $query_args );
        if ( !empty($categories) ) {
          if(is_array($categories)){
            foreach ( $categories as $category ) {
              if (is_object($category)) {
                $options[$category->name] = $category->slug;
              }
            }
          }
        }
        break;
        case 'products':
        case 'product':
        $loop = new WP_Query( $query_args );
        if ( !empty($loop) ) {
            while ( $loop->have_posts() ) : $loop->the_post(); 
              global $post; 
              if (is_object($post)) {
                $options[$post->post_title] = $post->ID;
              }
            endwhile; 
            wp_reset_postdata();
        } 
        break;
        case 'custom':
        case 'callback':
        if( is_callable( $query_args['function'] ) ) {
          $options = call_user_func( $query_args['function'], $query_args['args'] );
        }
        break;
      }
      return $options;
    }
}

//Load custom favicon.
if ( ! function_exists( 'floris_favicon_info' ) ) {
	function floris_favicon_info () {
		$fv = '';
		$fv = esc_url( FLORIS_THEME_DIRURI . 'assets/img/favicon.ico' );
    $favicon_info1_array = floris_get_option('favicon_info1');
		if ( isset( $favicon_info1_array['url'] ) && $favicon_info1_array['url'] != '' ) {
			$fv = $favicon_info1_array['url'] ; }
		if ( is_ssl() ) { $fv = str_replace( 'http://', 'https://', $fv ); }
	?>
		<link href="<?php print esc_url( $fv ); ?>" rel="shortcut icon" type="image/x-icon" />
	<?php
	} // End floris_favicon_info()
}

//Fixing the font size for the tag cloud widget.
add_filter( 'widget_tag_cloud_args', 'floris_repairwp_tag_cloud_args' );
if (!function_exists( 'floris_repairwp_tag_cloud_args')) {
	function floris_repairwp_tag_cloud_args($args) {
	$args['number'] = 10;
	$args['largest'] = 13;
	$args['smallest'] = 13;
	$args['unit'] = 'px';
	$args['format'] = 'list';
	return $args;
	}
}

// Adding span tag in category widgets for styling purpose
add_filter('wp_list_categories', 'floris_repairwp_cat_count');
function floris_repairwp_cat_count($links) {
  $links = str_replace('</a> (', ' <span>(', $links);
  $links = str_replace(')', ')</span></a>', $links);
  return $links;
}



/*-----------------------------------------------------------------------------------*/
/* Image Resizer script for on the fly resizing                                     */
/*-----------------------------------------------------------------------------------*/
// Source: https://github.com/syamilmj/Aqua-Resizer

if( ! class_exists('floris_opt_Aq_Resize') ) {

  /**
   * Image resizing class
   *
   * @since 1.0
   */
  class floris_opt_Aq_Resize {

    /**
     * The singleton instance
     */
    static private $instance = null;

    /**
     * No initialization allowed
     */
    private function __construct() {}

    /**
     * No cloning allowed
     */
    private function __clone() {}

    /**
     * For your custom default usage you may want to initialize an Aq_Resize object by yourself and then have own defaults
     */
    static public function getInstance() {
      if(self::$instance == null) {
        self::$instance = new self;
      }

      return self::$instance;
    }

    /**
     * Run, forest.
     */
    public function process( $url, $width = null, $height = null, $crop = null, $single = true, $upscale = true ) {

      // Validate inputs.
      if ( ! $url || ( ! $width && ! $height ) ) return false;

      $upscale = true;

      // Caipt'n, ready to hook.
      if ( true === $upscale ) add_filter( 'image_resize_dimensions', array($this, 'aq_upscale'), 10, 6 );

      // Define upload path & dir.
      $upload_info = wp_upload_dir();
      $upload_dir = $upload_info['basedir'];
      $upload_url = $upload_info['baseurl'];

      $http_prefix = "http://";
      $https_prefix = "https://";

      /* if the $url scheme differs from $upload_url scheme, make them match
         if the schemes differe, images don't show up. */
      if(!strncmp($url,$https_prefix,strlen($https_prefix))){ //if url begins with https:// make $upload_url begin with https:// as well
        $upload_url = str_replace($http_prefix,$https_prefix,$upload_url);
      }
      elseif(!strncmp($url,$http_prefix,strlen($http_prefix))){ //if url begins with http:// make $upload_url begin with http:// as well
        $upload_url = str_replace($https_prefix,$http_prefix,$upload_url);
      }


      // Check if $img_url is local.
      if ( false === strpos( $url, $upload_url ) ) return false;

      // Define path of image.
      $rel_path = str_replace( $upload_url, '', $url );
      $img_path = $upload_dir . $rel_path;

      // Check if img path exists, and is an image indeed.
      if ( ! file_exists( $img_path ) or ! getimagesize( $img_path ) ) return false;

      // Get image info.
      $info = pathinfo( $img_path );
      $ext = $info['extension'];
      list( $orig_w, $orig_h ) = getimagesize( $img_path );

      // Get image size after cropping.
      $dims = image_resize_dimensions( $orig_w, $orig_h, $width, $height, $crop );
      $dst_w = $dims[4];
      $dst_h = $dims[5];

      // Return the original image only if it exactly fits the needed measures.
      if ( ! $dims && ( ( ( null === $height && $orig_w == $width ) xor ( null === $width && $orig_h == $height ) ) xor ( $height == $orig_h && $width == $orig_w ) ) ) {
        $img_url = $url;
        $dst_w = $orig_w;
        $dst_h = $orig_h;
      } else {
        // Use this to check if cropped image already exists, so we can return that instead.
        $suffix = "{$dst_w}x{$dst_h}";
        $dst_rel_path = str_replace( '.' . $ext, '', $rel_path );
        $destfilename = "{$upload_dir}{$dst_rel_path}-{$suffix}.{$ext}";

        if ( ! $dims || ( true == $crop && false == $upscale && ( $dst_w < $width || $dst_h < $height ) ) ) {
          // Can't resize, so return false saying that the action to do could not be processed as planned.
          return $url;
        }
        // Else check if cache exists.
        elseif ( file_exists( $destfilename ) && getimagesize( $destfilename ) ) {
          $img_url = "{$upload_url}{$dst_rel_path}-{$suffix}.{$ext}";
        }
        // Else, we resize the image and return the new resized image url.
        else {

          $editor = wp_get_image_editor( $img_path );

          if ( is_wp_error( $editor ) || is_wp_error( $editor->resize( $width, $height, $crop ) ) )
            return $url;

          $resized_file = $editor->save();

          if ( ! is_wp_error( $resized_file ) ) {
            $resized_rel_path = str_replace( $upload_dir, '', $resized_file['path'] );
            $img_url = $upload_url . $resized_rel_path;
          } else {
            return $url;
          }

        }
      }

      // Okay, leave the ship.
      if ( true === $upscale ) remove_filter( 'image_resize_dimensions', array( $this, 'aq_upscale' ) );

      // Return the output.
      if ( $single ) {
        // str return.
        $image = $img_url;
      } else {
        // array return.
        $image = array (
          0 => $img_url,
          1 => $dst_w,
          2 => $dst_h
        );
      }

      return $image;
    }

    /**
     * Callback to overwrite WP computing of thumbnail measures
     */
    function aq_upscale( $default, $orig_w, $orig_h, $dest_w, $dest_h, $crop ) {
      if ( ! $crop ) return null; // Let the wordpress default function handle this.

      // Here is the point we allow to use larger image size than the original one.
      $aspect_ratio = $orig_w / $orig_h;
      $new_w = $dest_w;
      $new_h = $dest_h;

      if ( ! $new_w ) {
        $new_w = intval( $new_h * $aspect_ratio );
      }

      if ( ! $new_h ) {
        $new_h = intval( $new_w / $aspect_ratio );
      }

      $size_ratio = max( $new_w / $orig_w, $new_h / $orig_h );

      $crop_w = round( $new_w / $size_ratio );
      $crop_h = round( $new_h / $size_ratio );

      $s_x = floor( ( $orig_w - $crop_w ) / 2 );
      $s_y = floor( ( $orig_h - $crop_h ) / 2 );

      return array( 0, 0, (int) $s_x, (int) $s_y, (int) $new_w, (int) $new_h, (int) $crop_w, (int) $crop_h );
    }

  }

}

if ( ! function_exists('floris_opt_aq_resize') ) {

  /**
   * Resize an image using floris_opt_Aq_Resize Class
   *
   * @since 1.0
   *
   * @param string $url     The URL of the image
   * @param int    $width   The new width of the image
   * @param int    $height  The new height of the image
   * @param bool   $crop    To crop or not to crop, the question is now
   * @param bool   $single  If true only returns the URL, if false returns array
   * @param bool   $upscale If image not big enough for new size should it upscale
   * @return mixed If $single is true return new image URL, if it is false return array
   *               Array contains 0 = URL, 1 = width, 2 = height
   */
  function floris_opt_aq_resize( $url, $width = null, $height = null, $crop = null, $single = true, $upscale = false ) {
    $aq_resize = floris_opt_Aq_Resize::getInstance();
    return $aq_resize->process( $url, $width, $height, $crop, $single, $upscale );
  }

}

//Catalog mode hooks
function catalog_mode() {
  if (floris_get_option('catalog_mode')) {
    remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart');
    function custom_my_account_menu_items( $items ) {
      unset($items['orders']);
      unset($items['downloads']);
      unset($items['edit-address']);
      return $items;
    }
    add_filter( 'woocommerce_account_menu_items', 'custom_my_account_menu_items' );
  }
}
add_filter( 'init', 'catalog_mode' ); 

// Get Banner Grid image
function get_grid_image($image, $image_size){
  $size_array = explode( 'x', $image_size );
  $size       = count( $size_array ) == 1 ? $image_size : $size_array;

  if ( function_exists( 'wpb_getImageBySize' ) ) {
    $image = wpb_getImageBySize( array(
      'attach_id'  => $image,
      'thumb_size' => $size,
    ) );

    $image = $image['thumbnail'];
  } else {
    $image = wp_get_attachment_image_src( $atts['image'], $size );
    if ( $image ) {
      $image = sprintf( '<img alt="%s" src="%s">',
        esc_attr( $text ),
        esc_url( $image[0] ) 
      );
    }
  }
  return $image;
}

/*
Function that will set infinite scrolling to be displayed in the page.
*/
function set_infinite_scrolling(){
    if(!is_singular() && floris_get_option('prod_shop_inf_scroll')){
    ?>    
      <script type="text/javascript">  
        if (jQuery('.pagination-box').length) {
          jQuery('.wrapper-post .infinity-container').infiniteScroll({
            path: '.pagination-box .next-page',
            append: '.wrapper-post .product',
            status: '.page-load-status',
            hideNav: '.pagination-box',
            scrollThreshold: 200,
            history: false,
              onInit: function() {
                this.on( 'append', function() {
                  jQuery('.page-load-status').show();
                  setTimeout(function() {
                    jQuery('.infinity-container .product').not('animated').each(function(index){
                      jQuery(this).css('transition-delay', '0.'+index+'s').addClass('animated');
                    });
                  }, 500);
                  setTimeout(function() {
                    jQuery('.page-load-status').hide();
                  }, 1000);
                })
              }
          });
        }          
      </script>        
    <?php
    }
}
add_action( 'wp_footer', 'set_infinite_scrolling', 100 );

/*-----------------------------------------------------------------------------------*/
/* END */
/*-----------------------------------------------------------------------------------*/